#!/bin/sh
# Copyright (c) 2002, Intel Corporation. All rights reserved.
# Created by:  rolla.n.selbak REMOVE-THIS AT intel DOT com
# This file is licensed under the GPL license.  For the full content
# of this license, see the COPYING file at the top level of this
# source tree.


# Test pthread_key_delete
#
#  If successful, it returns zero.

# This is tested implicitly via assertion 1 & 2.

echo "Tested implicitly via assertion 1 & 2.  See output for status."
exit 0
